// VGA.C (C) 2000 Jay Cook
// VGA graphics routines
#include <stdio.h>
#include <stdlib.h>
#include <dos.h>

#define VIDEO_INT           0x10      /* the BIOS video interrupt. */
#define WRITE_DOT           0x0C      /* BIOS func to plot a pixel. */
#define SET_MODE            0x00      /* BIOS func to set the video mode. */
#define VGA_256_COLOR_MODE  0x13      /* use to set 256-color mode. */
#define TEXT_MODE           0x03      /* use to set 80x25 text mode. */

#define SCREEN_WIDTH        320       /* width in pixels of mode 0x13 */
#define SCREEN_HEIGHT       200       /* height in pixels of mode 0x13 */
#define NUM_COLORS          256       /* number of colors in mode 0x13 */

typedef unsigned char  byte;
typedef unsigned short word;
typedef unsigned long  dword;

byte Color[768];
byte *VGA2=(byte *)0xA0000000L;        /* this points to video memory. */
byte *VGA;                            // The double buffer
byte *HiddenC;
word *my_clock=(word *)0x0000046C;    /* this points to the 18.2hz system
                                         clock. */
int n2;

void set_mode(byte mode)
{ // Sets video mode
  union REGS regs;

  regs.h.ah = SET_MODE;
  regs.h.al = mode;
  int86(VIDEO_INT, &regs, &regs);
}

void CLS()
{
 memset(VGA,0,64000);
}

void DBuffer()
{
// memcpy (VGA,VGA,64000);
}

/////////////////////////////
void SetBuffer()
/////////////////////////////
{  //SetBuffer -=- sets up buffer
  if ((VGA = (byte *) malloc(65000)) == NULL)
  {
    QuitDos(1,"Not enough memory for double buffer.","none");
  }
  if ((HiddenC=(byte *) malloc(100)) == NULL)
  {
    QuitDos(1,"Not enough memory for double buffer.","none");
  }
  memset(VGA,0,64000);
  memset(HiddenC,0,100);
}

/////////////////////////////
void PlotPixel(int x, int y)
/////////////////////////////
{
 VGA[y*320+x]=15; 
}

/////////////////////////////
void DrawBlock(int x, int y,int xl, int yl, int BColor)
/////////////////////////////
{ // Draws a solid block on screen
  int x2, y2;
  for(y2=0; y2<yl; y2++)
   for(x2=0; x2<xl; x2++)
    VGA[(y2+y)*320+(x2+x)]=BColor; 
}

/////////////////////////////
void DrawLine(int x, int y, int xl, int BColor)
/////////////////////////////
{
 int x2,y2;
 for(y2=0; y2<xl; y2++) VGA[((y2+y)*320)+x]=BColor;
}


/////////////////////////////
void SpriteGet(int x,int y, char *SpriteG)
//////////////////////////
{  // gets font graphics from bsave file
 int x2,y2,x3,y3,snum;
 snum=0;
 for (y2=0; y2< 8; y2++)
 {
  for(x2=0; x2< 8; x2++)
  {
   x3=x2+x;
   y3=y2+y;
   //SpriteG[snum]=VGA[ (((y2 + y3)<<8) + ((y2 + y3)<<6)) + (x2 + x3)];
   SpriteG[snum]=VGA[y3*320+x3];
   snum++;
  }
 }
}

/////////////////////////////
void SpritePut(int x, int y, char *SpriteG)
/////////////////////////////
{
 int SNum, x2,y2,x3,y3;
 SNum=0;
 for(y2=0;y2<8;y2++)
 {
  for(x2=0;x2<8;x2++)
  {
   x3=x2+x;
   y3=y2+y;
   VGA[y3*320+x3]=SpriteG[SNum];
   SNum++;
  }
 }
}

/////////////////////////////
void SpriteDraw(int x, int y, char *SpriteG)
/////////////////////////////
{
 int SNum, x2,y2,x3,y3;
 SNum=0;
 for(y2=0;y2<8;y2++)
 {
  for(x2=0;x2<8;x2++)
  {
   x3=x2+x;
   y3=y2+y;
   if(SpriteG[SNum] > 0) 
   { VGA[y3*320+x3]=SpriteG[SNum]; }
   SNum++;
  }
 }
}

/////////////////////////////
void SpriteDrawBG(int x, int y, char *SpriteG)
/////////////////////////////
{
 int SNum, x2,y2,x3,y3;
 SNum=0;
 for(y2=0;y2<8;y2++)
 {
  for(x2=0;x2<8;x2++)
  {
   x3=x2+x;
   y3=y2+y;
   if(SpriteG[SNum] > 0 && x3 > -1 && x3 < 320 && y3 > -1 && y3 < 200)
     VGA[y3*320+x3]=SpriteG[SNum];
   SNum++;
  }
 }
}

/////////////////////////////
void wait()
//////////////////////////
{  // wait Wait for a specified number of clock ticks (18hz).
 word start;
 start=*my_clock;
 while (*my_clock-start<1)
 {
 }
//    {*my_clock=*my_clock;}/* this line is for some compilers that would otherwise ignore this loop */
}

////////////////////////
void LoadBMP(dword gfx1[11])
//////////////////////////
{  // Loads a bsave file
 FILE *fptr;
 int Bvalue[1];
 unsigned int n;

 // Checks to see if file is there
 if( (fptr=fopen(gfx1,"rb"))==NULL)
  QuitDos(1,"Graphic file not found!",gfx1);

 // If file is bad
 if( fseek(fptr, 7, 0) !=0)
  QuitDos(1,"Error in graphic file!",gfx1);

 for(n=0;n<64000;n++)
 {  // Gets the bitmap
  fread(&Bvalue,1,1,fptr);
  VGA[n]=Bvalue[0];
 }
 for(n=0;n<768;n++)
 {  // Gets the palette
  fread(&Bvalue,1,1,fptr);
  Color[n]=Bvalue[0];
 }  // Close file(duh!)
 fclose(fptr);

   // Set palette
 outp(0x03c8,0);              /* tell the VGA that palette data					 is coming. */
 for(n=0;n<768;n++)
  outp(0x03c9,Color[n]);
}

/////////////////////////////
void ScrollPal()
//////////////////////////
{  // Rolls part of the palette to give special effects like waterfall
 int k, SColor1,SColor2;
 SColor1=160;
 SColor2=176;

 for(k=SColor2;k>SColor1;k--)
 {  // Rotates colors
  Color[(k+1)*3-3]=Color[k*3-3];
  Color[(k+1)*3-2]=Color[k*3-2];
  Color[(k+1)*3-1]=Color[k*3-1];
 } // Moves the last color to the front
  Color[(SColor1+1)*3-3]=Color[SColor2*3-3];
  Color[(SColor1+1)*3-2]=Color[SColor2*3-2];
  Color[(SColor1+1)*3-1]=Color[SColor2*3-1];
// Writes data to VGA
 outp(0x03c8,0);             /* tell the VGA that palette data is coming. */
 for(k=0;k<768;k++)
  outp(0x03c9,Color[k]);

}

/////////////////////////////
void WaitVR()
//////////////////////////
{  // Waits for Verticle Retrace
  asm mov dx,0x03DA
  l1: asm {
    in      al,dx;
    and     al,0x08;
    jnz     l1;
  }
  l2: asm {
    in      al,dx;
    and     al,0x08;
    jz      l2;
  }
 memcpy (VGA2,VGA,64000);
}
/////////////////////////////
void CopyChar1()
/////////////////////////////
{ // copies char into memory for scrollie
 int x3,y3,x2,y2, snum;
 snum=0;
 for (y2=0; y2< 8; y2++)
 {
  for(x2=0; x2< 8; x2++)
  {
   x3=x2+312-16;
   y3=y2+184-8;
   HiddenC[snum]=VGA[y3*320+x3];
   snum++;
  }
 }
   DrawBlock(296,176,8,8,0);
 n2=0;
}

/////////////////////////////
void CheckPixel(int x, int y)
//////////////////////////
{
 if(VGA[y*320+x]==32) return(1);
}
